/**
 * Nested Semicircle Animation
 * 
 * Copyright (c) 2008 Tom Beddard
 * http://www.subblue.com
 * 
 * Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
 * 
 * Credits:
 * Original Java implementation found here
 * 	 http://groups.google.com/group/sci.fractals/browse_thread/thread/2265d921ad137788
 */

package {
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.utils.*;
	
	public class Main extends Sprite
	{
		private var sprite:Sprite;
		private var _depth:int = 5;				// Current nesting depth 
		private var _nesting:int = 2;			// Number of nested semicircles
		private var radius:Number;				// Radius of first circle
		private var angle:Number = 0;			// Incremental angle
		private var offsetLeft:Number = 0;		// Left offset
		private var offsetRight:Number = 0;		// Right offset
		private var deg2rad = Math.PI / 180;	// For converting degrees to radians
		private var arcStep = 5;				// Step in degrees for arcs
		
		
		public function Main()
		{
			stage.scaleMode = "noScale";
			stage.align = "TL";
			init();
		}	
		
		
		/**
		 *	Initialisation
		 */
		private function init():void
		{
			if (stage.stageWidth)
			{
				sprite = new Sprite();
				addChild(sprite);

				sprite.x = stage.stageWidth / 2;
				sprite.y = stage.stageHeight / 2;
				radius = (Math.min(stage.stageWidth,stage.stageHeight) / 2) - 20;

				stage.addEventListener(MouseEvent.MOUSE_MOVE, mouseMoveListener);
				stage.addEventListener(Event.ENTER_FRAME, enterFrameListener);
			} else {
				setTimeout(init, 100);
			}
		}
		
		
		/**
		 *	Update the nested semicircle
		 */
		private function enterFrameListener(e:Event):void
		{
			sprite.graphics.clear();
			drawSemiCircles(0, 0, radius, angle++, 0);
		}
		
		
		/**
		 *	Set nesting depth according to vertical mouse position
		 */
		private function mouseMoveListener(e:MouseEvent):void
		{
			_depth = 1 + Math.ceil((stage.mouseY / stage.stageHeight) * 6);
			_nesting = 1 + Math.ceil((stage.mouseX / stage.stageWidth) * 5);
		}
		
		
		/**
		 *	Recursively draw the semicircles
		 */
        private function drawSemiCircles(x0:Number, y0:Number, r:Number, a:int, iter:int):void
        {
			if (r < 1) return;
			
			sprite.graphics.beginFill(0x000000);
            drawSlice(x0, y0, r, a, 180);
			sprite.graphics.endFill();
			
			sprite.graphics.beginFill(0xFFFFFF);
            drawSlice(x0, y0, r, a + 180, 180);
			sprite.graphics.endFill();
			
            if (iter < _depth)
            {
				var d:Number;
				var rn:Number = r / _nesting;
				var xa:Number = Math.cos(a * deg2rad);
				var ya:Number = Math.sin(a * deg2rad);
				
				for (var i:int = 0; i < _nesting; i++)
				{
					d = (i * (2 * rn)) - rn * (_nesting - 1);
					drawSemiCircles(x0 + (d * xa), y0 + (d * ya), rn, angle + a, iter + 1);
				}
            }
        }
		
		
		/**
		 *	Draw a circle slice
		 *	x0				slice center x position
		 *	y0				slice center y position
		 *	r				radius
		 *	startingAngle	in degrees, 0 = 3 o'clock
		 *	arcAngle		in degrees, draws clockwise.
		 */
		private function drawSlice(x0:Number, y0:Number, r:Number, startAngle:Number, arcAngle:Number):void
		{
			var angle:Number;
			var steps:Number = arcAngle / arcStep;
			var x1 = x0 + Math.cos(startAngle * deg2rad) * r;
			var y1 = y0 + Math.sin(startAngle * deg2rad) * r;
			
			sprite.graphics.moveTo(x0, y0);
			sprite.graphics.lineTo(x1, y1);
			
			for (var i:int = 1; i <= steps; i++)
			{
				angle = (startAngle + i * arcStep) * deg2rad;
				x1 = x0 + Math.cos(angle) * r;
				y1 = y0 + Math.sin(angle) * r;
				sprite.graphics.lineTo(x1, y1);
			}
			sprite.graphics.lineTo(x0, y0);
		}
	}
}
